/***
   NAME
     sbcm
   DESCRIPTION
     EBT implementation of the basic size-structured consumer-resource model
     used in chapter 9 of:

     Population and Community Ecology of Ontogenetic Development
     Andr M. de Roos & Lennart Persson
     Monographs in Population Biology 51, Princeton University Press (2013)

   OUTPUT VARIABLES:

     OUTPUT VARIABLES OF THE STRUCTURED MODEL
      0: R
      1: Juvenile biomass
      2: Adult biomass
      3: Total biomass
      4: Juvenile number
      5: Adult number
      6: Total number
      7: ingest_R
      8: nu_J
      9: nu_A
     10: mort_J
     11: mort_A
     12: birthrate (biomass)
     13: maturation rate (biomass)
     14: adult lifespan
     15: adult fecundity (numbers)
     16: total population birth rate (numbers)
     17: total population maturation rate (numbers)
     18: Juvenile period
     19: Juvenile survival

   Last modification: AMdR - Nov 09, 2013
***/
#include  "escbox.h"

#define LOGISTIC	0		// 0: Semi-chemostat; 1: Logistic


/*
 *==========================================================================
 *
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *
 *==========================================================================
 */
#define time		env[ 0]
#define R		env[ 1]

#define TotalBirths	env[ 2]

#define	age		i_state(0)
#define	mass		i_state(1)

#define maturity	i_const(0)
#define IDnbegin	i_const(1)

#if (EVENT_NR > 0)
#define ismature(n)	((popIDcard[0][n][maturity] > 0.5))
#else
#define TINY		1.0E-5
#define ismature(n)	((popIDcard[0][n][maturity] > 0.5) || (log(pop[0][n][mass]) > logsm*(1.0 - TINY)))
// #define ismature(n)	((popIDcard[0][n][maturity] > 0.5) || (log(pop[0][n][mass]) >= logsm))
#endif

/*
 *==========================================================================
 *
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *
 *==========================================================================
 */

#define RMAX		parameter[ 0]
#define DELTA		parameter[ 1]

#define SB		parameter[ 2]
#define SM		parameter[ 3]

#define	IMAX		parameter[ 4]
#define Q		parameter[ 5]
#define	H		parameter[ 6]
#define	T		parameter[ 7]

#define SIGMA		parameter[ 8]

#define	MUJ		parameter[ 9]
#define	MUA		parameter[10]

#define LAKEVOLUME	1.0
#define ZERODENSITY	1.0E-9



/*
 *==========================================================================
 *
 *		OTHER DEFINITIONS
 *
 *==========================================================================
 */

static double		Adu_bio, Juv_bio, Adu_nr, Juv_nr;

static double		ingest_R;
static double		nu_J, nu_A, nuplus_J, nuplus_A;
static double		mort_J, mort_A;
static double		birthrate, MatRate, maturation;
static double		logz, logsm;
static double		JuvPeriod = 0.0, JuvSurv = 0.0;


/*
 *==========================================================================
 *
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *
 *==========================================================================
 */

void	UserInit(int argc, char **argv, double *env, population *pop)

{
  int			i;

  logz  = log(SM/SB);
  logsm = log(SM);

  // Set the maturity flags
  for(i=0, Adu_bio=0.0, Juv_bio=0.0; i<cohort_no[0]; i++)
    {
      if((pop[0][i][mass] >= SM) || isequal(log(pop[0][i][mass]), logsm))
	{
	  popIDcard[0][i][maturity] = 1.0;

	  Adu_bio += pop[0][i][number]*pop[0][i][mass];
	}
      else
	{
	  popIDcard[0][i][maturity] = 0.0;

	  Juv_bio += pop[0][i][number]*pop[0][i][mass];
	}

      if (iszero(popIDcard[0][i][IDnbegin])) popIDcard[0][i][IDnbegin] = pop[0][i][number];
    }

#if (LOGISTIC > 0)
  ReportNote("Logistic resource dynamics");
#else
  ReportNote("Semi-chemostat resource dynamics");
#endif
  ReportNote("\n    Ratio of offspring-adult body size: z = %4.2f\n", exp(-logz));

#if (BIFURCATION ==1)
#if (ADJUST_CL == 1)
  ReportNote("Cohort limit adjusted to target %4d cohorts", TARGET_COHORTS);
  initMeasureBifstats(argv[1], 1);
#else
  ReportNote("Cohort limit constant at %.3f", cohort_limit);
  initMeasureBifstats(argv[1], 0);
#endif

#endif // BIFURCATION

  return;
}



/*
 *==========================================================================
 *
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *
 *==========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)

{
  logz  = log(SM/SB);
  logsm = log(SM);

  bpoint_no[0] = 1;

  return;
}



/*==========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)

{
  bpoints[0][0][mass] = SB;
  bpoints[0][0][age]  = 0.0;

  TotalBirths = 0.0;
  MatRate     = 0.0;

  if (pop[0][cohort_no[0]-1][age] > 10*cohort_limit)
    SetStepSize(0.01*cohort_limit);

  return;
}

/*==========================================================================*/
#define MAX_EXP		50.0

inline double   bexp(double x)

{
  double                pw = x;

  pw = max(pw, -MAX_EXP);
  pw = min(pw,  MAX_EXP);

  return exp(pw);
}




/*==========================================================================*/

void	SetStructVars(double *env, population *pop, population *ofs)

{
  int			i;
  double		FR;

  Juv_nr    = 0.0;
  Adu_nr    = 0.0;
  Juv_bio   = 0.0;
  Adu_bio   = 0.0;

  for(i=0; i<cohort_no[0]; i++)
    {
      if (ismature(i))
      	{
	  Adu_nr  += pop[0][i][number];
	  Adu_bio += pop[0][i][mass]*pop[0][i][number];
	}
      else
        {
	  Juv_nr  += pop[0][i][number];
	  Juv_bio += pop[0][i][mass]*pop[0][i][number];
        }
    }

  // Contribution of newborns to biomass and foraging
  if (ofs)
    {
      Juv_nr  += ofs[0][0][number];
      Juv_bio += ofs[0][0][mass] + SB*ofs[0][0][number];
    }

  FR	    = R/(H + R);
  ingest_R  = 0.0;

  nu_J      = SIGMA  *IMAX*FR - T;
  ingest_R += Juv_bio;

  nu_A      = SIGMA*Q*IMAX*FR - T;
  ingest_R += Q*Adu_bio;

  ingest_R *= IMAX*FR;

  nuplus_J  = max(nu_J, 0.0);
  nuplus_A  = max(nu_A, 0.0);

  mort_J    = MUJ;
  mort_A    = MUA;

  mort_J   -= min(nu_J, 0.0);
  mort_A   -= min(nu_A, 0.0);

  birthrate = nuplus_A*Adu_bio;

  return;
}


/*
 *==========================================================================
 *
 *			SPECIFICATION OF DERIVATIVES
 *
 *==========================================================================
 */

void	Gradient(double *env,     population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)

{
  int			i;

#if (BIFURCATION == 1)
//  if (rk_level == 1) measureMinMax(env, pop);
#endif

  SetStructVars(env, pop, ofs);

  for(i=0; i<cohort_no[0]; i++)
    {
      popgrad[0][i][number] = -1.0;
      popgrad[0][i][mass]   = -1.0;
      popgrad[0][i][age]    = 1.0;

      // We assume that individuals never shrink in mass
      // Starvation is dealt with as additional mortality
      if (ismature(i))
	{
	  popgrad[0][i][number]  = -mort_A*pop[0][i][number];
	  popgrad[0][i][mass]    = 0.0;
	}
      else
	{
	  popgrad[0][i][number] = -mort_J*pop[0][i][number];
	  popgrad[0][i][mass]   = nuplus_J*pop[0][i][mass];
	}
    }

  // The derivatives for the boundary cohort

  ofsgrad[0][0][number] = birthrate/SB  	     - mort_J*ofs[0][0][number];
  ofsgrad[0][0][age]    = ofs[0][0][number]          - mort_J*ofs[0][0][age];
  ofsgrad[0][0][mass]   = ofs[0][0][number]*nuplus_J*SB - mort_J*ofs[0][0][mass];
  ofsgrad[0][0][mass]  += nuplus_J*ofs[0][0][mass];

  envgrad[0] = 1.0;
#if (LOGISTIC > 0)
  envgrad[1] = DELTA*R*(1.0 - R/RMAX) - ingest_R/LAKEVOLUME;
#else
  envgrad[1] = DELTA*(RMAX - R)       - ingest_R/LAKEVOLUME;
#endif
  envgrad[2] = birthrate/SB;

  return;
}



/*
 *==========================================================================
 *
 *	SPECIFICATION OF EVENT LOCATION AND DYNAMIC COHORT CLOSURE
 *
 *==========================================================================
 */

void	EventLocation(double *env, population *pop, population *ofs,
		      population *bpoints, double *events)

{
  int			i;

  events[0] = -1.0;
  for (i=0; i<cohort_no[0]; i++)
    {
      if (popIDcard[0][i][maturity] < 0.5)      // if juvenile
	events[0] = max(events[0], log(pop[0][i][mass]) - logsm);
    }

  return;
}




/*==============================================================================*/

int	ForceCohortEnd(double *env, population *pop, population *ofs,
		       population *bpoints)

{
  int			i;

  for(i=0; i<cohort_no[0]; i++)
    {
      if (ismature(i)) continue;
      if((pop[0][i][mass] >= SM) || isequal(log(pop[0][i][mass]), logsm))
	{
	  popIDcard[0][i][maturity] = 1.0;
	  JuvPeriod = pop[0][i][age];
	  JuvSurv   = pop[0][i][number]/popIDcard[0][i][IDnbegin];
	  MatRate   = pop[0][i][number]*SM/cohort_limit;
	}
    }
  SetStepSize(0.1*cohort_limit);

  return NO_COHORT_END;
}



/*
 *==========================================================================
 *
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *
 *==========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)

{
  int			i;
  int			first_adult = -1;
  double		total_adults = 0.0, total_age = 0.0;

  ofsIDcard[0][0][IDnbegin] = TotalBirths;

  for(i=0; i<cohort_no[0]; i++)
    {
      if((popIDcard[0][i][maturity] > 0.5) || (pop[0][i][mass] >= SM) || isequal(log(pop[0][i][mass]), logsm))
	{
	  popIDcard[0][i][maturity] = 1.0;
	}

      if (ismature(i))
	{
	  if (first_adult == -1) first_adult = i;

	  total_adults      += pop[0][i][number];
	  total_age         += pop[0][i][age]*pop[0][i][number];
	  pop[0][i][number]  = 0.0;

	  pop[0][first_adult][number] 	  = total_adults;
	  pop[0][first_adult][mass]   	  = SM;
	  pop[0][first_adult][age]    	  = total_age/total_adults;

	  popIDcard[0][first_adult][maturity] = 1.0;
	}
      else if (pop[0][i][number] < ZERODENSITY*popIDcard[0][i][IDnbegin])
	pop[0][i][number] = 0.0;
    }

  return;
}



/*
 *==========================================================================
 *
 *			SPECIFICATION OF OUTPUT VARIABLES
 *
 *==========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)

{
  int			outnr = 0;

  LabelState(0, "T = %.2f", env[0]);

  SetStructVars(env, pop, NULL);

  output[outnr++] = R;
  output[outnr++] = Juv_bio/LAKEVOLUME;
  output[outnr++] = Adu_bio/LAKEVOLUME;
  output[outnr++] = (Juv_bio + Adu_bio)/LAKEVOLUME;
  output[outnr++] = Juv_nr/LAKEVOLUME;
  output[outnr++] = Adu_nr/LAKEVOLUME;
  output[outnr++] = (Juv_nr + Adu_nr)/LAKEVOLUME;

  output[outnr++] = ingest_R/LAKEVOLUME;
  output[outnr++] = nu_J;
  output[outnr++] = nu_A;
  output[outnr++] = mort_J;
  output[outnr++] = mort_A;
  output[outnr++] = birthrate/LAKEVOLUME;	// Total biomass reproduction rate
  output[outnr++] = MatRate/LAKEVOLUME;		// Total biomass maturation rate

  output[outnr++] = 1/mort_A;			// Adult lifespan
  output[outnr++] = nuplus_A*SM/SB;		// Fecundity (in number of offspring!!)

  output[outnr++] = (birthrate/SB)/LAKEVOLUME;	// Total birth rate (in number of offspring!!)
  output[outnr++] = (MatRate/SM)/LAKEVOLUME;	// Total maturation rate (in number of offspring!!)

  output[outnr++] = JuvPeriod;
  output[outnr++] = JuvSurv;

  return;
}



/*==============================================================================*/
